/*
 * Created on Dec 6, 2006
 * Created by reisdorf1
 */
package gov.llnl.ics.pe.mapguide.query;


import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import net.sf.json.util.JSONDynaBean;

import org.apache.log4j.Logger;
import org.osgeo.mapguide.MgAgfReaderWriter;
import org.osgeo.mapguide.MgBooleanProperty;
import org.osgeo.mapguide.MgByteProperty;
import org.osgeo.mapguide.MgByteReader;
import org.osgeo.mapguide.MgByteSource;
import org.osgeo.mapguide.MgClassDefinition;
import org.osgeo.mapguide.MgCoordinate;
import org.osgeo.mapguide.MgCoordinateCollection;
import org.osgeo.mapguide.MgCoordinateSystem;
import org.osgeo.mapguide.MgCoordinateSystemFactory;
import org.osgeo.mapguide.MgCoordinateSystemTransform;
import org.osgeo.mapguide.MgCreateSdfParams;
import org.osgeo.mapguide.MgDataPropertyDefinition;
import org.osgeo.mapguide.MgDeleteFeatures;
import org.osgeo.mapguide.MgDoubleProperty;
import org.osgeo.mapguide.MgException;
import org.osgeo.mapguide.MgFeatureCommandCollection;
import org.osgeo.mapguide.MgFeatureGeometricType;
import org.osgeo.mapguide.MgFeaturePropertyType;
import org.osgeo.mapguide.MgFeatureQueryOptions;
import org.osgeo.mapguide.MgFeatureReader;
import org.osgeo.mapguide.MgFeatureSchema;
import org.osgeo.mapguide.MgFeatureService;
import org.osgeo.mapguide.MgFeatureSpatialOperations;
import org.osgeo.mapguide.MgGeometricEntity;
import org.osgeo.mapguide.MgGeometricPropertyDefinition;
import org.osgeo.mapguide.MgGeometry;
import org.osgeo.mapguide.MgGeometryFactory;
import org.osgeo.mapguide.MgGeometryProperty;
import org.osgeo.mapguide.MgInsertFeatures;
import org.osgeo.mapguide.MgInt16Property;
import org.osgeo.mapguide.MgInt32Property;
import org.osgeo.mapguide.MgInt64Property;
import org.osgeo.mapguide.MgLayer;
import org.osgeo.mapguide.MgLayerCollection;
import org.osgeo.mapguide.MgLinearRing;
import org.osgeo.mapguide.MgMap;
import org.osgeo.mapguide.MgPoint;
import org.osgeo.mapguide.MgPolygon;
import org.osgeo.mapguide.MgProperty;
import org.osgeo.mapguide.MgPropertyCollection;
import org.osgeo.mapguide.MgPropertyDefinition;
import org.osgeo.mapguide.MgPropertyDefinitionCollection;
import org.osgeo.mapguide.MgPropertyType;
import org.osgeo.mapguide.MgResourceIdentifier;
import org.osgeo.mapguide.MgResourceService;
import org.osgeo.mapguide.MgSelection;
import org.osgeo.mapguide.MgServiceType;
import org.osgeo.mapguide.MgSingleProperty;
import org.osgeo.mapguide.MgSiteConnection;
import org.osgeo.mapguide.MgSpatialContextReader;
import org.osgeo.mapguide.MgStringProperty;
import org.osgeo.mapguide.MgUserInformation;

/*
 *  This code is mainly just a Java representation of the GT example from the MapGuide Live Application Gallery (http://data.mapguide.com/mapguide/gt/index.php)
 * 
 * @author reisdorf1
 * 
 */
public class Query {

    private static Logger log = Logger.getLogger(Query.class.getName());

    private Map args;

    private MgSiteConnection site;

    public static final String[] numOperators = new String[] { "Equal to", "Not equal to", "Greater than","Greater than or equal to", "Less than", "Less than or equal to" };
    public static final String[] numExpressions = new String[] { " = %s", " != %s", " > %s", " >= %s", " < %s", " <= %s" };
    public static final String[] strOperators = new String[] { "Begins with", "Contains", "Equal to" };
    public static final String[] strExpressions = new String[] { " like '%s%%'", " like '%%%s%%'", " = '%s'" };

    public Query(Map args) throws MgException {
        super();
        this.args = args;
        this.site = new MgSiteConnection();
        site.Open(new MgUserInformation((String) args.get("SESSION")));
    }

    /**
     * Return a list of LabelValue Beans representing the Layers Physical name and LegendLabel name
     * 
     * @return
     * @throws MgException
     */
    public List getMapLayerNames() throws MgException {
        List layerList = new ArrayList();
        MgResourceService resourceService = (MgResourceService) site.CreateService(MgServiceType.ResourceService);
        MgMap map = new MgMap();
        map.Open(resourceService, (String) args.get("MAPNAME"));

        MgLayer layer = null;
        for (int i = 0; i < map.GetLayers().GetCount(); i++) {
            MgLayer nextLayer = map.GetLayers().GetItem(i);
            layerList.add(new LabelValue(nextLayer.GetLegendLabel(), nextLayer.GetName()));
        }
        return layerList;

    }

    public List getLayerProperties() throws MgException {
        String mapName = (String) args.get("MAPNAME");
        String layerName = (String) args.get("LAYERNAME");
        return getLayerProperties(layerName, mapName);
    }

    private List getLayerProperties(String layerName, String mapName) throws MgException {

        List properties = new ArrayList();
        MgResourceService resourceService = (MgResourceService) site.CreateService(MgServiceType.ResourceService);
        MgMap map = new MgMap();
        map.Open(resourceService, mapName);
        MgLayer layer = map.GetLayers().GetItem(layerName);

        MgFeatureService featureService = (MgFeatureService) site.CreateService(MgServiceType.FeatureService);
        MgResourceIdentifier resId = new MgResourceIdentifier(layer.GetFeatureSourceId());
        String[] schemaClass = layer.GetFeatureClassName().split(":");
        String mgSchema = "";
        String mgClass = "";
        if (schemaClass.length == 1) {
            mgSchema = schemaClass[0];
            mgClass = schemaClass[0];
        } else {
            mgSchema = schemaClass[0];
            mgClass = schemaClass[1];
        }
        MgClassDefinition classDef = featureService.GetClassDefinition(resId, mgSchema, mgClass);

        for (int i = 0; i < classDef.GetProperties().GetCount(); i++) {
            MgPropertyDefinition propertyDef = classDef.GetProperties().GetItem(i);
            if (propertyDef.GetPropertyType() == MgFeaturePropertyType.DataProperty) {
                MgDataPropertyDefinition dataPropertyDef = (MgDataPropertyDefinition) propertyDef;
                int dataType = dataPropertyDef.GetDataType();
                if (isValidDataType(dataType)) {
                    properties.add(new Property(propertyDef.GetName(), dataType == MgPropertyType.String));
                }
            }
        }
        Collections.sort(properties);
        return properties;
    }

    public boolean toggleSpatialFilter() throws MgException {

        boolean result = true;

        MgResourceService resourceService = (MgResourceService) this.site.CreateService(MgServiceType.ResourceService);

        MgMap map = new MgMap();
        map.Open(resourceService, (String) this.args.get("MAPNAME"));

        MgLayerCollection layers = map.GetLayers();
        if (layers.Contains("_QuerySpatialFilter")) {
            MgLayer layer = layers.GetItem("_QuerySpatialFilter");
            if (this.args.get("VISIBLE").equals("true"))
                layer.SetVisible(true);
            else
                layer.SetVisible(false);

            map.Save(resourceService);
        }

        return result;
    }

    /**
     * Adds the Layer to the map representing the Spatial Intesection the Filter should use.
     * 
     * @return
     * @throws MgException
     * @throws IOException
     */
    public boolean showSpatialFilter() throws MgException, IOException {
        boolean result = true;
        MgResourceIdentifier sdfResId = new MgResourceIdentifier("Session:" + (String) this.args.get("SESSION")
                + "//Filter.FeatureSource");

        MgResourceService resourceService = (MgResourceService) this.site.CreateService(MgServiceType.ResourceService);
        MgFeatureService featureService = (MgFeatureService) this.site.CreateService(MgServiceType.FeatureService);

        MgFeatureCommandCollection updateCommands = new MgFeatureCommandCollection();

        MgMap map = new MgMap();
        map.Open(resourceService, (String) this.args.get("MAPNAME"));

        MgLayer layer = null;
        MgLayerCollection layers = map.GetLayers();
        if (layers.Contains("_QuerySpatialFilter")) {
            layer = layers.GetItem("_QuerySpatialFilter");
            updateCommands.Add(new MgDeleteFeatures("Filter", "ID like '%'"));
        } else {
            // Create the Feature Source (SDF)

            MgFeatureSchema sdfSchema = createFilterSchema();
            MgCreateSdfParams sdfParams = new MgCreateSdfParams("MAPCS", map.GetMapSRS(), sdfSchema);
            featureService.CreateFeatureSource(sdfResId, sdfParams);

            // Create the Layer

            MgResourceIdentifier layerResId = new MgResourceIdentifier("Session:" + this.args.get("SESSION")
                    + "//Filter.LayerDefinition");

            String layerDefinition = FileUtils.getFileContents("filterlayerdefinition.templ");
            layerDefinition = String.format(layerDefinition, sdfResId.ToString());

            MgByteSource byteSource = new MgByteSource(layerDefinition.getBytes(), layerDefinition.length()); // ,
            // layerDefinition.length());
            resourceService.SetResource(layerResId, byteSource.GetReader(), null);

            layer = new MgLayer(layerResId, resourceService);

            layer.SetName("_QuerySpatialFilter");
            layer.SetLegendLabel("_QuerySpatialFilter");
            layer.SetDisplayInLegend(false);
            layer.SetSelectable(false);

            layers.Insert(0, layer);
        }

        // Make the layer visible

        layer.SetVisible(true);
        map.Save(resourceService);

        // Add the geometry to the filter feature source

        MgPolygon polygon = createPolygonFromGeomText((String) this.args.get("GEOMTEXT"));
        MgAgfReaderWriter agfWriter = new MgAgfReaderWriter();
        MgByteReader byteReader = agfWriter.Write(polygon);

        MgPropertyCollection propertyValues = new MgPropertyCollection();
        propertyValues.Add(new MgGeometryProperty("Geometry", byteReader));

        updateCommands.Add(new MgInsertFeatures("Filter", propertyValues));

        featureService.UpdateFeatures(sdfResId, updateCommands, false);

        return result;

    }

    /**
     * Executes the Query
     * 
     * @return
     * @throws MgException
     */
    public List execute() throws MgException {
        List results = new ArrayList();
        boolean noColonInSchema = false;
        MgResourceService resourceService = (MgResourceService) site.CreateService(MgServiceType.ResourceService);
        MgMap map = new MgMap();
        map.Open(resourceService, (String) args.get("MAPNAME"));
        MgLayer layer = map.GetLayers().GetItem((String) args.get("LAYERNAME"));

        MgFeatureService featureService = (MgFeatureService) site.CreateService(MgServiceType.FeatureService);
        MgResourceIdentifier resId = new MgResourceIdentifier(layer.GetFeatureSourceId());

        String featureClass = layer.GetFeatureClassName();
        String featureGeometry = layer.GetFeatureGeometryName();

        String[] schemaAndClass = featureClass.split(":");

        String mgSchema = "";
        String mgClass = "";

        if (schemaAndClass.length == 1) {
            mgSchema = schemaAndClass[0];
            mgClass = schemaAndClass[0];
            noColonInSchema = true;
        } else {
            mgSchema = schemaAndClass[0];
            mgClass = schemaAndClass[1];
        }

        // Initialize the Coordinate System Transformation
        MgClassDefinition classDef = featureService.GetClassDefinition(resId, mgSchema, mgClass);
        MgGeometricPropertyDefinition geomProp = (MgGeometricPropertyDefinition) classDef.GetProperties().GetItem(
                featureGeometry);
        String spatialContext = geomProp.GetSpatialContextAssociation();

        MgCoordinateSystemTransform csTransform = null;
        MgCoordinateSystemTransform csInverseTransform = null;
        MgCoordinateSystemFactory coordSysFactory = new MgCoordinateSystemFactory();

        MgSpatialContextReader scReader = featureService.GetSpatialContexts(resId, false);

        while (scReader.ReadNext() && csTransform == null) {

            if (scReader.GetName().equalsIgnoreCase(spatialContext)) {
                MgCoordinateSystem source = coordSysFactory.Create(scReader.GetCoordinateSystemWkt());
                MgCoordinateSystem target = coordSysFactory.Create(map.GetMapSRS());
                csTransform = new MgCoordinateSystemTransform(source, target);
                csInverseTransform = new MgCoordinateSystemTransform(target, source);
            }
        }
        scReader.Close();

        // Execute the Query
        int queryMax = Integer.parseInt((String) args.get("QUERYMAX"));
        MgFeatureQueryOptions queryOptions = new MgFeatureQueryOptions();

        if (args.get("USEPROPERTYFILTER").equals("true")) {
            String propertyFilter = (String) args.get("PROPERTYNAME");
            int operatorArrayIndex = Integer.parseInt((String) args.get("OPERATOR"));
            if (args.get("ISSTRING").equals("true")) {
                propertyFilter += String.format(strExpressions[operatorArrayIndex], (String) args.get("VALUE"));
            } else {
                propertyFilter += String.format(numExpressions[operatorArrayIndex], (String) args.get("VALUE"));
            }

            queryOptions.SetFilter(propertyFilter);

        } // end if usePropertyFilter

        if (((String) args.get("USESPATIALFILTER")).equals("true")) {
            // SPATIAL FILTER STUFF
            MgPolygon polygon = createPolygonFromGeomText((String) args.get("GEOMTEXT"));
            MgGeometricEntity entity = polygon.Transform(csInverseTransform);
            MgGeometry geometry = (MgGeometry) entity;
            queryOptions.SetSpatialFilter(featureGeometry, geometry, MgFeatureSpatialOperations.Intersects);
        }

        int count = 0;
        MgAgfReaderWriter geometryReaderWriter = new MgAgfReaderWriter();

        String featureClassName = layer.GetFeatureClassName();
        if (noColonInSchema) {
            featureClassName += ":" + featureClassName;
        }
        MgFeatureReader featureReader = featureService.SelectFeatures(resId, featureClassName, queryOptions);
        while (featureReader.ReadNext() && (queryMax <= 0 || count < queryMax)) {
            String displayValue = getFeaturePropertyValue(featureReader, (String) args.get("OUTPUTPROPERTY"));

            MgByteReader byteReader = featureReader.GetGeometry(featureGeometry);
            MgGeometry geometry = geometryReaderWriter.Read(byteReader);
            MgGeometricEntity centerPoint = geometry.GetCentroid();
            centerPoint = centerPoint.Transform(csTransform);

            // Store the List of Ids as Feature Objects
            List idList = getFeatureIdList(map, layer, featureReader);
            results.add(new Feature(displayValue, (MgPoint) centerPoint.GetCentroid(), idList));
            count++;

        }

        return results;
    }

    /**
     * Return the XML String for a given FeatureId
     * 
     * @return
     * @throws MgException
     */
    public String GetSelectionXML() throws MgException {
        MgResourceService resourceService = (MgResourceService) site.CreateService(MgServiceType.ResourceService);
        MgFeatureService featureService = (MgFeatureService) this.site.CreateService(MgServiceType.FeatureService);

        MgMap map = new MgMap();
        map.Open(resourceService, (String) args.get("MAPNAME"));
        MgLayer layer = map.GetLayers().GetItem((String) args.get("LAYERNAME"));
        MgResourceIdentifier resId = new MgResourceIdentifier(layer.GetFeatureSourceId());
        String featureClass = layer.GetFeatureClassName();
        String[] schemaAndClass = featureClass.split(":");

        String mgSchema = "";
        String mgClass = "";

        if (schemaAndClass.length == 1) {
            mgSchema = schemaAndClass[0];
            mgClass = schemaAndClass[0];
        } else {
            mgSchema = schemaAndClass[0];
            mgClass = schemaAndClass[1];
        }

        MgClassDefinition classDef = featureService.GetClassDefinition(resId, mgSchema, mgClass);

        MgPropertyCollection properties = new MgPropertyCollection();
        // Get the FeatureId list JSON object
        String ids = (String) args.get("IDLIST");
        JSONArray jsonArray = JSONArray.fromObject(ids);
        List idList = JSONArray.toList(jsonArray);
        for (Iterator iter = idList.iterator(); iter.hasNext();) {
            Object o = iter.next();
            // Get the FeatureId object from the JSON Element
            JSONDynaBean element = (JSONDynaBean) o;
            JSONObject jsonObject = JSONObject.fromDynaBean(element);
            FeatureId featureId = (FeatureId) JSONObject.toBean(jsonObject, FeatureId.class);

            String key = featureId.getName();
            String value = featureId.getValue();
            int type = featureId.getType();

            MgProperty property = getFeaturePropertyForType(type, key, value);
            if (property != null) {
                properties.Add(property);
            } else {
                log.warn("Null property for " + featureId);
            }

        }

        MgSelection selection = new MgSelection(map);
        selection.AddFeatureIds(layer, featureClass, properties);
        String xml = selection.ToXml();
        return xml;
    }

    /**
     * Return a List of <code>FeatureId</code> Objects stroing the Feature Name, MgPropertyType,
     * and String Value of the Feature
     * 
     * @param map
     * @param layer
     * @param featureReader
     * @return
     * @throws MgException
     */
    private List getFeatureIdList(MgMap map, MgLayer layer, MgFeatureReader featureReader) throws MgException {

        MgClassDefinition classDef = featureReader.GetClassDefinition();
        MgPropertyDefinitionCollection idProps = classDef.GetIdentityProperties();
        List idList = new ArrayList();

        for (int i = 0; i < idProps.GetCount(); i++) {
            MgPropertyDefinition idProp = idProps.GetItem(i);

            if (idProp instanceof MgDataPropertyDefinition) {

                MgDataPropertyDefinition dataPropertyDef = (MgDataPropertyDefinition) idProp;
                int dataType = dataPropertyDef.GetDataType();

                switch (dataType) {
                case MgPropertyType.Boolean:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.Boolean, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.Byte:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.Byte, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.Single:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.Single, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.Double:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.Double, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.Int16:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.Int16, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.Int32:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.Int32, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.Int64:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.Int64, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.String:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.String, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.DateTime:
                    idList.add(new FeatureId(idProp.GetName(), MgPropertyType.DateTime, getFeaturePropertyValue(
                            featureReader, idProp.GetName())));
                    break;

                case MgPropertyType.Null:
                case MgPropertyType.Blob:
                case MgPropertyType.Clob:
                case MgPropertyType.Feature:
                case MgPropertyType.Geometry:
                case MgPropertyType.Raster:
                    break;
                }
            }

        }
        return idList;
    }

    /**
     * Return the proper MgProperty Object for the given String value and type.
     * 
     * @param mgPropertyType
     * @param key
     * @param value
     * @return
     * @throws MgException
     */
    private MgProperty getFeaturePropertyForType(int mgPropertyType, String key, String value) throws MgException {
        MgProperty prop = null;
        switch (mgPropertyType) {
        case MgPropertyType.Boolean:
            boolean b = new Boolean(value).booleanValue();
            prop = new MgBooleanProperty(key, b);
            break;
        case MgPropertyType.Byte:
            short s = new Short(value).shortValue();
            prop = new MgByteProperty(key, s);
            break;
        case MgPropertyType.Single:
            float f = new Float(value).floatValue();
            prop = new MgSingleProperty(key, f);
            break;
        case MgPropertyType.Double:
            double d = new Double(value).doubleValue();
            prop = new MgDoubleProperty(key, d);
            break;
        case MgPropertyType.Int16:
            short s1 = new Short(value).shortValue();
            prop = new MgInt16Property(key, s1);
            break;
        case MgPropertyType.Int32:
            int i = new Integer(value).intValue();
            prop = new MgInt32Property(key, i);
            break;
        case MgPropertyType.Int64:
            long l = new Long(value).longValue();
            prop = new MgInt64Property(key, l);
            break;
        case MgPropertyType.String:
            prop = new MgStringProperty(key, value);
            break;
        case MgPropertyType.DateTime:
        case MgPropertyType.Null:
        case MgPropertyType.Blob:
        case MgPropertyType.Clob:
        case MgPropertyType.Feature:
        case MgPropertyType.Geometry:
        case MgPropertyType.Raster:
            break;
        }
        return prop;
    }

    /**
     * For a given MgProperty, return the Value as a String
     * 
     * @param featureReader
     * @param name
     * @return
     * @throws MgException
     */
    private String getFeaturePropertyValue(MgFeatureReader featureReader, String name) throws MgException {
        String value = "";
        int propertyType = featureReader.GetPropertyType(name);
        switch (propertyType) {
        case MgPropertyType.Boolean:
            boolean b = featureReader.GetBoolean(name);
            value = Boolean.toString(b);
            break;

        case MgPropertyType.Byte:
            short s = featureReader.GetByte(name);
            value = Short.toString(s);
            break;

        case MgPropertyType.Single:
            float f = featureReader.GetSingle(name);
            value = Float.toString(f);
            break;

        case MgPropertyType.Double:
            double d = featureReader.GetDouble(name);
            value = Double.toString(d);
            break;

        case MgPropertyType.Int16:
            short s1 = featureReader.GetInt16(name);
            value = Short.toString(s1);
            break;

        case MgPropertyType.Int32:
            int i = featureReader.GetInt32(name);
            value = Integer.toString(i);
            break;

        case MgPropertyType.Int64:
            long l = featureReader.GetInt64(name);
            value = Long.toString(l);
            break;

        case MgPropertyType.String:
            value = featureReader.GetString(name);
            break;

        case MgPropertyType.DateTime:
        case MgPropertyType.Null:
        case MgPropertyType.Blob:
        case MgPropertyType.Clob:
        case MgPropertyType.Feature:
        case MgPropertyType.Geometry:
        case MgPropertyType.Raster:
            value = "[unsupported data type]";
            break;
        }
        return value;
    }

    private boolean isValidDataType(int dataType) {
        boolean valid = true;

        switch (dataType) {
        case MgPropertyType.Blob:
            valid = false;
            break;
        case MgPropertyType.Clob:
            valid = false;
            break;
        case MgPropertyType.Byte:
            valid = false;
            break;
        case MgPropertyType.Feature:
            valid = false;
            break;
        case MgPropertyType.Geometry:
            valid = false;
            break;
        case MgPropertyType.Null:
            valid = false;
            break;
        }
        return valid;
    }

    private MgFeatureSchema createFilterSchema() throws MgException {
        MgFeatureSchema filterSchema = new MgFeatureSchema();
        filterSchema.SetName("FilterSchema");

        MgClassDefinition filterClass = new MgClassDefinition();
        filterClass.SetName("Filter");
        MgPropertyDefinitionCollection properties = filterClass.GetProperties();

        MgDataPropertyDefinition idProperty = new MgDataPropertyDefinition("ID");
        idProperty.SetDataType(MgPropertyType.Int32);
        idProperty.SetReadOnly(true);
        idProperty.SetNullable(false);
        idProperty.SetAutoGeneration(true);
        properties.Add(idProperty);

        MgGeometricPropertyDefinition geometryProperty = new MgGeometricPropertyDefinition("Geometry");
        geometryProperty.SetGeometryTypes(MgFeatureGeometricType.Surface);
        geometryProperty.SetHasElevation(false);
        geometryProperty.SetHasMeasure(false);
        geometryProperty.SetReadOnly(false);
        geometryProperty.SetSpatialContextAssociation("MAPCS");
        properties.Add(geometryProperty);

        filterClass.GetIdentityProperties().Add(idProperty);
        filterClass.SetDefaultGeometryPropertyName("Geometry");

        filterSchema.GetClasses().Add(filterClass);

        return filterSchema;
    }

    private MgPolygon createPolygonFromGeomText(String geomText) throws MgException {
        MgGeometryFactory geometryFactory = new MgGeometryFactory();

        String[] vertices = geomText.split(",");
        int count = Integer.parseInt(vertices[0]);

        MgCoordinateCollection coords = new MgCoordinateCollection();
        for (int i = 0; i < count; i++) {
            double x = Double.parseDouble(vertices[(i * 2) + 1]);
            double y = Double.parseDouble(vertices[(i * 2) + 2]);
            MgCoordinate coord = geometryFactory.CreateCoordinateXY(x, y);
            coords.Add(coord);
        }

        MgLinearRing linearRing = geometryFactory.CreateLinearRing(coords);
        MgPolygon polygon = geometryFactory.CreatePolygon(linearRing, null);

        return polygon;
    }

}
