/* 
 * Created by reisdorf1
 */
package gov.llnl.ics.pe.mapguide.web;

import gov.llnl.ics.pe.mapguide.query.Query;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;

/**
 * Simple Controller for the Query Object.  
 * Logic was taken from the PHP example from the GT example from the MapGuide Live Application Gallery (http://data.mapguide.com/mapguide/gt/index.php)
 * This uses the Spring framework, but the Controller can be changed to use/return your controllers object type (i.e. Struts, or Jsp)
 * @author reisdorf1
 * 
 */
public class QueryController extends AbstractFicController{
    
    protected Map getArgs(HttpServletRequest request) throws Exception{
        Map args = new HashMap();
        String sessionId = ServletRequestUtils.getRequiredStringParameter(request, "SESSION");
        String mapName = request.getParameter("MAPNAME");
        args.put("SESSION", sessionId);
        args.put("MAPNAME",mapName);
        args.put("LAYERNAME",request.getParameter("LAYERNAME") );
        args.put("QUERYMAX",request.getParameter("QUERYMAX") );
        args.put("USEPROPERTYFILTER",request.getParameter("USEPROPERTYFILTER") );
        args.put("PROPERTYNAME",request.getParameter("PROPERTYNAME") );
        args.put("OPERATOR",request.getParameter("OPERATOR") );
        args.put("VALUE",request.getParameter("VALUE") );
        args.put("USESPATIALFILTER",request.getParameter("USESPATIALFILTER") );
        args.put("GEOMTEXT", request.getParameter("GEOMTEXT"));
        args.put("COMMAND", request.getParameter("COMMAND"));
        args.put("ISSTRING", request.getParameter("ISSTRING"));
        args.put("OUTPUTPROPERTY", request.getParameter("OUTPUTPROPERTY"));
        args.put("IDLIST",request.getParameter("IDLIST"));
        args.put("VISIBLE",request.getParameter("VISIBLE"));
        return args;
    }
    
    
    public ModelAndView handleRequestInternal(HttpServletRequest request, HttpServletResponse response) throws Exception {
        
        
        // MapUtils.InitializeWebTier(request);
        Map args = getArgs(request);
        
        Query query = new Query(args);
        
        
        List layerList = query.getMapLayerNames();
        JSONArray strOps = JSONArray.fromArray(query.strOperators);
        JSONArray numOps = JSONArray.fromArray(query.numOperators);
        request.setAttribute("strOps",strOps);
        request.setAttribute("numOps",numOps);
        request.setAttribute("layerList", layerList);
        
        response.setContentType("text/plain");
        response.setHeader("Cache-Control", "no-cache");
        String cmd = (String) args.get("COMMAND");
        if (cmd != null && !cmd.equalsIgnoreCase(""))
        {
           
            if(cmd.equalsIgnoreCase("GETLAYERPROPERTIES")) {
                    List list = query.getLayerProperties();
                    JSONArray jsonArray = JSONArray.fromObject(list);
                    response.getWriter().write(jsonArray.toString());
                    // response = json->encode(query->GetLayerProperties());
            }
            else if(cmd.equalsIgnoreCase("TOGGLESPATIALFILTER")) {
            boolean b = query.toggleSpatialFilter();
            JSONObject jsonObject = new JSONObject(b);
            response.getWriter().write(jsonObject.toString());
            
            }
            else if(cmd.equalsIgnoreCase("SHOWSPATIALFILTER")) {
                boolean b  = query.showSpatialFilter();
                JSONObject jsonObject = new JSONObject(b);
                response.getWriter().write(jsonObject.toString());
            }
            else if(cmd.equalsIgnoreCase("EXECUTEQUERY")) {
                List list = query.execute();                
                JSONArray jsonArray = JSONArray.fromObject(list);
                response.getWriter().write(jsonArray.toString());
            }
            else if(cmd.equalsIgnoreCase("GETSELECTIONXML")) {
                String selection = query.GetSelectionXML();                
                response.getWriter().write(selection);
            }
            return null;
            }
        else {
            return new ModelAndView(getSuccessView(), args);
            
        }


        
        
    }

}
